﻿using System;
using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text.RegularExpressions;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Options;

namespace GpsNose.SDK.Framework
{
    /// <summary>
    /// Provides an item-initialized caching wrapper base functionality.
    /// </summary>
    /// <remarks>
    /// If the GetCachedItem() doesn't find the item, it initializes it.
    /// </remarks>
    /// <example>
    /// Can be used for derivation like MyCache : GnCache
    /// string MyCache.FullName { get => GetCachedItem("fullName", 1 hour, () => { return $"{person.FirstName} {person.LastName}" })
    /// </example>
    public class GnCache
    {
        protected static readonly Dictionary<string, object> _locks;
        protected static readonly GnMemoryCache _cache;

        static GnCache()
        {
            _cache = new GnMemoryCache(new MemoryCacheOptions());
            _locks = new Dictionary<string, object>();
        }


        public static TResult GetCachedItem<TResult>(string propName, TimeSpan expiriesIn, Func<TResult> initFunc)
        {
            // already cached?
            if (_cache.TryGetValue(propName, out var obj))
                return (TResult)obj;

            // lock exists already?
            if (!_locks.ContainsKey(propName))
                _locks.Add(propName, new object());

            lock (_locks[propName])
            {
                // still not set? threading..
                if (!_cache.TryGetValue(propName, out obj))
                {
                    obj = initFunc();
                    _cache.Set(propName, obj, DateTimeOffset.Now.AddTicks(expiriesIn.Ticks));
                }

                return (TResult)obj;
            }
        }


        /// <summary>
        /// Clears the items from the cache.
        /// </summary>
        /// <param name="keyPattern">The items to be cleared by a regexp-pattern of the key. If null, everything is removed.</param>
        public static void ClearCache(string keyPattern = null, bool isRegexPattern = true)
        {
            if (string.IsNullOrWhiteSpace(keyPattern))
            {
                // remove all items
                foreach (var key in _cache.Keys())
                    _cache.Remove(key);

                return;
            }

            // regex?
            if (isRegexPattern)
            {
                // remove key-patterns only
                var regex = new Regex(keyPattern);
                foreach (var key in _cache.Keys())
                    if (regex.IsMatch(key))
                        _cache.Remove(key);

                return;
            }

            // otherwise: raw key
            _cache.Remove(keyPattern);
        }
    }


    public class GnMemoryCache : MemoryCache
    {
        public IEnumerable<string> Keys()
        {
            var entries = GetType().BaseType.GetField("_entries", BindingFlags.NonPublic | BindingFlags.Instance).GetValue(this);
            var keys = (IEnumerable<object>)entries.GetType().GetProperty("Keys", BindingFlags.Instance | BindingFlags.Public).GetValue(entries);

            foreach (var key in keys)
                yield return (string)key;
        }

        public GnMemoryCache(IOptions<MemoryCacheOptions> optionsAccessor) : base(optionsAccessor)
        {
        }
    }
}
