﻿using System;
using System.Collections.Generic;
using GpsNose.SDK.Mashup.Model;

namespace GpsNose.SDK.Mashup.Api.Modules
{
    /// <summary>
    /// Used for managing your mashups.
    /// </summary>
    /// <remarks>
    /// You can register the mashup, verify it, regenerate its app-key, add/del sub-communities etc.
    /// </remarks>
    public class GnAdminApi : GnApiModuleBase
    {
        private static TimeSpan _adminTtlCacheSecs = new TimeSpan(0, 0, 1);

        protected override string ControllerBasePath => "MashupAdmin";

        internal GnAdminApi(GnLoginApiAdmin loginApi) : base(loginApi)
        {
        }


        /// <summary>
        /// Registers a new web-community.
        /// </summary>
        /// <remarks>
        /// Every web-community is in the form: %|@|*www.mydomain.com, where the prefix specifies the privacy level:
        /// % = public mashup: anybody can join, anybody can read community data
        /// @ = closed mashup: invitation needed for join, but anybody can read community data
        /// * = private mashup: invitation needed for join, only members can read community data
        /// Important: the mashup's privacy level can ONLY be set here and can't be changed later!
        /// </remarks>
        /// <param name="tag">The community's tag, prefixed with the privacy level.</param>
        /// <returns>Validation key, which must be validated for the new web-community.</returns>
        public string RegisterCommunityWeb(string tag)
        {
            if (string.IsNullOrWhiteSpace(tag))
                throw new ArgumentException("tag required");

            var result = SimpleResultCall<string>("RegisterCommunityWeb", new { tag }, "ValidationKey", TimeSpan.MaxValue);
            return result;
        }


        /// <summary>
        /// Regenerates the app-key for a web-community.
        /// </summary>
        /// <param name="tag">The community tag.</param>
        /// <returns>The new app-key</returns>
        public string RegenerateAppKey(string tag)
        {
            if (string.IsNullOrWhiteSpace(tag))
                throw new ArgumentException("tag required");

            var result = SimpleResultCall<string>("RegenerateAppKey", new { tag }, "AppKey", TimeSpan.MaxValue);
            return result;
        }


        /// <summary>
        /// Validates a web-community.
        /// </summary>
        /// <remarks>
        /// Before a web-community can be used, it must be validated.
        /// The validation key is provided while a web-community has been created with RegisterCommunityWeb().
        /// The key can be placed into:
        /// a.) your website's default home document as a meta-tag named "gpsnose-validation-key" (and content is the key), or
        /// b.) a file named as the key: without any extension or with .txt|.htm |.html extensions
        /// </remarks>
        /// <param name="tag">Which web-community is to be validated.</param>
        /// <returns>The app-key for the validated web-community.</returns>
        public string ValidateCommunityWeb(string tag)
        {
            if (string.IsNullOrWhiteSpace(tag))
                throw new ArgumentException("tag required");

            var result = SimpleResultCall<string>("ValidateCommunityWeb", new { tag }, "AppKey", TimeSpan.MaxValue);
            return result;
        }


        /// <summary>
        /// Adds a sub-community to an existing validated web-community.
        /// </summary>
        /// <remarks>
        /// A connected website could have more sub-communities, like @www.geohamster.com@admins.
        /// A sub-community prefix defines the community protection-level: %=public, @=closed, *=private
        /// </remarks>
        /// <param name="tag">The sub-community tag in the form: %www.mydomain.com@subcommunityname</param>
        /// <param name="acls">The new sub-community will get these ACLs when access by members.</param>
        public void AddSubCommunity(string tag, GnCommunityAcl acls = GnCommunityAcl.CommentsFromMembers | GnCommunityAcl.ListMembers | GnCommunityAcl.MembersInviteMembers)
        {
            if (string.IsNullOrWhiteSpace(tag))
                throw new ArgumentException("tag required");

            ExecuteCall("AddSubCommunity", new
            {
                tag = tag,
                acls = acls
            }, cacheTtl: TimeSpan.MaxValue);
        }


        /// <summary>
        /// Deletes an existing sub-community from a web-community.
        /// </summary>
        /// <param name="tag">The sub-community tag, like *www.geohamster.com@level-10</param>
        public void DelSubCommunity(string tag)
        {
            if (string.IsNullOrWhiteSpace(tag))
                throw new ArgumentException("tag required");

            ExecuteCall("DelSubCommunity", new { tag }, cacheTtl: TimeSpan.MaxValue);
        }


        /// <summary>
        /// Updates a web-community.
        /// </summary>
        /// <param name="tag">The main web-community, like %www.mydomain.com</param>
        /// <param name="hosts">Optional: allowed calling hosts. If null: will be removed.</param>
        /// <param name="mashupTokenCallbackUrl">Optional: which mashup callback-url to call sync immediately while scanning the token. If null: will be removed.</param>
        public void UpdateCommunityWeb(string tag, List<string> hosts = null, string mashupTokenCallbackUrl = null)
        {
            if (string.IsNullOrWhiteSpace(tag))
                throw new ArgumentException("tag required");

            ExecuteCall("UpdateCommunityWeb", new { tag, hosts, mashupTokenCallbackUrl }, cacheTtl: TimeSpan.MaxValue);
        }


        /// <summary>
        /// Gets the user's mashups.
        /// </summary>
        /// <returns>A list of successfully registered and validated mashups.</returns>
        /// <remarks>
        /// Mashups waiting for validation are not returned: these do NOT belong the user until they have been successfully validated.
        /// </remarks>
        public List<GnMashup> GetOwnMashups()
        {
            var result = ExecuteCall<List<GnMashup>>("GetOwnMashups", cacheTtl: _adminTtlCacheSecs);
            return result;
        }
    }
}
