﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Text;
using System.Threading;
using GpsNose.SDK.Framework;
using GpsNose.SDK.Framework.Json;
using GpsNose.SDK.Framework.Logging;
using GpsNose.SDK.Mashup.Framework;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;


namespace GpsNose.SDK.Mashup.Api.Modules
{
    /// <summary>
    /// Internal base-class for any concrete API module.
    /// </summary>
    public abstract class GnApiModuleBase
    {
        private const int QUOTA_WAIT_SECS_AFTER_EXCEEDED = 2;
        private const int QUOTA_MAX_CALLS = 3;
        private int DEFAULT_CACHE_TTL_MINUTES = 15;

        private static int _quota_CallCounter = 0;
        private static DateTime _quota_LastCall = DateTime.MinValue;

        internal GnLoginApiBase LoginApi { get; }

        protected virtual string ControllerBasePath => "MashupApi";


        protected virtual void ClearCacheForActionName(string actionName)
        {
            string keyPattern = $"{ControllerBasePath}/{actionName}";
            GnCache.ClearCache(keyPattern);
        }


        protected virtual void ClearCacheForActionNames(string[] actionNames)
        {
            foreach (string actionName in actionNames)
                ClearCacheForActionName(actionName);
        }


        /// <summary>
        /// Construct the module with the loginApi in param.
        /// </summary>
        /// <param name="loginApi"></param>
        protected GnApiModuleBase(GnLoginApiBase loginApi)
        {
            // PARANOIA: can't be null normally..
            LoginApi = loginApi ?? throw new GnException("loginApi must be provided");
        }


        /// <summary>
        /// Construct the module with the LoginApi set to this instance.
        /// </summary>
        protected GnApiModuleBase()
        {
            // notice: only GnLoginApi may call this constructor!
            LoginApi = (GnLoginApiBase)this;
        }


        /// <summary>
        /// Call the server API with a known generic response-type.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="actionName"></param>
        /// <param name="request"></param>
        /// <param name="ignoreNotFound"></param>
        /// <param name="cacheTtl"></param>
        /// <returns></returns>
        protected virtual T ExecuteCall<T>(string actionName, object request = null, bool ignoreNotFound = false, TimeSpan cacheTtl = new TimeSpan())
        {
            var result = ExecuteCall(actionName, request, typeof(T), ignoreNotFound, cacheTtl);

            return (T)result;
        }


        /// <summary>
        /// Call the server API with a response-type as a parameter.
        /// </summary>
        /// <param name="actionName"></param>
        /// <param name="request"></param>
        /// <param name="responseType"></param>
        /// <param name="ignoreNotFound"></param>
        /// <param name="cacheTtl"></param>
        /// <returns></returns>
        protected virtual object ExecuteCall(string actionName, object request, Type responseType = null, bool ignoreNotFound = false, TimeSpan cacheTtl = new TimeSpan())
        {
            HandleCallQuota();

            object result;

            if (responseType == null)
                responseType = typeof(object);

            var ser = new JsonTool();
            string resJson = null;
            string cacheKey = null;

            try
            {
                using (WebClient wc = new WebClient())
                {
                    var reqExp = request.ToExpando();

                    // create the request-URL
                    string url = $"{GnApi.ApiRoot}/{ControllerBasePath}/{actionName}";
                    var urlParams = new Dictionary<string, string>();
                    // ..login needed?
                    if (LoginApi.IsLoggedIn == true)
                        urlParams.Add("lid", LoginApi.LoginId);
                    // ..language
                    urlParams.Add("lang", LoginApi.LangId);
                    url = GnUtility.GetQueryStringFromKeyVals(url, urlParams);

                    // app-key needed?
                    if (!string.IsNullOrWhiteSpace(LoginApi.AppKey))
                        reqExp.appKey = LoginApi.AppKey;

                    string reqJson = ser.SerializeToString(reqExp);

                    // setup cache
                    cacheKey = $"{url}#{reqJson}";
                    if (cacheTtl.Ticks == 0)
                        cacheTtl = TimeSpan.FromMinutes(DEFAULT_CACHE_TTL_MINUTES);

                    // use cache|POST to read the result
                    byte[] resData = cacheTtl.Ticks == TimeSpan.MaxValue.Ticks ?
                        ExecutePost(url, reqJson, wc) :
                        GnCache.GetCachedItem(cacheKey, cacheTtl, () => ExecutePost(url, reqJson, wc));

                    if (resData.Length == 0)
                    {
                        ResetCachedResut();
                        return null;
                    }

                    resJson = Encoding.UTF8.GetString(resData);

                    try
                    {
                        result = string.IsNullOrWhiteSpace(resJson) ? null : ser.DeserializeFromString(resJson, responseType);
                    }
                    catch (JsonReaderException)
                    {
                        // try to deser into general json, if possible
                        try
                        {
                            object jObject = ser.DeserializeFromString(resJson);
                            AssureNotJsonError(jObject);
                        }
                        catch (JsonReaderException)
                        {
                            // if it's not JSON, it's ok
                        }

                        // return raw data
                        return resData;
                    }
                    catch (JsonSerializationException)
                    {
                        // retry with object-type
                        result = ser.DeserializeFromString(resJson, typeof(object));
                    }
                }
            }
            catch (WebException ex) when (ex.Message == "Unable to connect to the remote server")
            {
                ResetCachedResut();

                throw new GnException("can't connect to the server");
            }
            catch (Exception ex)
            {
                ResetCachedResut();

                // some API calls return 404 by design, so it's ok..
                if (ex is WebException && ex.Message == "The remote server returned an error: (404) Not Found." && ignoreNotFound)
                    return null;

                // otherwise: it's an critical error
                GnLogger.LogException(ex);
                throw new GnException($"error in server API call: {ex.Message}");
            }

            // ok or err?
            if (result is JObject jo)
                AssureNotJsonError(jo);
            else if (result != null)
            {
                // the result was a specialized type: try to parse the original errCode/message
                var jObject = ser.DeserializeFromString(resJson);
                AssureNotJsonError(jObject);
            }

            return result;


            void AssureNotJsonError(object someResult)
            {
                if (!(someResult is JObject jsonResult) || !jsonResult.HasValues)
                    return;

                var errCode = jsonResult.GetValue("ErrorCode");
                if (errCode?.Type == JTokenType.Integer && errCode.Value<int>() != 0)
                {
                    ResetCachedResut();

                    string msg = jsonResult.GetValue("Message")?.Value<string>();
                    throw new GnException($"GpsNose API error: {msg}");
                }
            }


            byte[] ExecutePost(string postUrl, string postJson, WebClient postWc)
            {
                postWc.Headers[HttpRequestHeader.ContentType] = "application/json";
                postWc.Encoding = Encoding.UTF8;

                byte[] reqData = Encoding.UTF8.GetBytes(postJson);
                return postWc.UploadData(postUrl, reqData);
            }


            void ResetCachedResut()
            {
                GnCache.ClearCache(cacheKey, false);
            }
        }


        /// <summary>
        /// To avoid unneeded traffic to the server, slow-down a bit.
        /// </summary>
        /// <remarks>
        /// Server checks this anyway, so to preserve the client's traffic, avoid excessive usage.
        /// </remarks>
        private void HandleCallQuota()
        {
            var secsFromLastCall = DateTime.Now.Subtract(_quota_LastCall).TotalSeconds;

            if (secsFromLastCall > QUOTA_WAIT_SECS_AFTER_EXCEEDED)
            {
                // time-frame is ok
                _quota_CallCounter = 0;
                GnLogger.Verbose($"resetting counter, sec={secsFromLastCall}");
            }
            else
            {
                // time-frame too short
                _quota_CallCounter++;
                GnLogger.Verbose($"counter={_quota_CallCounter}");
                if (_quota_CallCounter >= QUOTA_MAX_CALLS)
                {
                    // counter exceeded: wait a bit..
                    GnLogger.Verbose("counter exceeded, wating..");
                    Thread.Sleep(QUOTA_WAIT_SECS_AFTER_EXCEEDED * 1000);
                    _quota_CallCounter = 0;
                }
            }

            _quota_LastCall = DateTime.Now;
        }


        /// <summary>
        /// Call the server with a simple return-type in a named result-property.
        /// </summary>
        /// <typeparam name="TResultProp"></typeparam>
        /// <param name="actionName"></param>
        /// <param name="request"></param>
        /// <param name="resultPropName"></param>
        /// <returns></returns>
        protected virtual TResultProp SimpleResultCall<TResultProp>(string actionName, object request, string resultPropName, TimeSpan cacheTtl = new TimeSpan())
        {
            var response = ExecuteCall(actionName, request, cacheTtl: cacheTtl);

            var result = ((JObject)response).Value<TResultProp>(resultPropName);

            return result;
        }
    }
}
