﻿using System;
using System.Collections.Generic;
using GpsNose.SDK.Mashup.Framework;
using GpsNose.SDK.Mashup.Model;

namespace GpsNose.SDK.Mashup.Api.Modules
{
    /// <summary>
    /// Comments reading/writing API.
    /// </summary>
    public class GnCommentsApi : GnApiModuleBase
    {
        private static readonly string[] CLEAR_CACHE_PATTERNS = { "GetCommentsPage" };

        internal GnCommentsApi(GnLoginApiBase loginApi) : base(loginApi)
        {
        }


        /// <summary>
        /// Gets the comments page for a specified item type/key, or for the sub/community itself
        /// </summary>
        /// <param name="itemKey">The commented item-key; if commenting Community, it may be left out for the main-community or specified with the sub-community.</param>
        /// <param name="itemType">The commented item-type for the item-key.</param>
        /// <param name="pageSize">Page size. Default 10, or [1..20].</param>
        /// <param name="lastKnownTicks">To start paging, leave out or set to current or future UTC. Later use the last-known ticks from the last page.</param>
        public List<GnComment> GetCommentsPage(string itemKey = null, GnCommentItemType itemType = GnCommentItemType.Community, int? pageSize = null, long lastKnownTicks = GnSettings.FAR_FUTURE_TICKS)
        {
            var result = ExecuteCall<List<GnComment>>("GetCommentsPage", new
            {
                itemType = (int)itemType,
                itemKey = itemKey,
                pageSize = pageSize,
                lastKnownTicks = lastKnownTicks
            });

            return result;
        }


        /// <summary>
        /// Adds a new comment.
        /// </summary>
        /// <param name="text">The comment text to be added.</param>
        /// <param name="itemKey">The commented item-key; if commenting Community, it may be left out for the main-community or specified with the sub-community.</param>
        /// <param name="itemType">The commented item-type for the item-key.</param>
        /// <returns>The created-comment's creation-ticks, for reference in update/delete/UI etc.</returns>
        public long AddComment(string text, string itemKey = null, GnCommentItemType itemType = GnCommentItemType.Community)
        {
            var creation = ExecuteCall<long>("AddComment", new
            {
                itemType = (int)itemType,
                itemKey = itemKey,
                text = text
            }, cacheTtl: TimeSpan.MaxValue);

            ClearCacheForActionNames(CLEAR_CACHE_PATTERNS);

            return creation;
        }


        /// <summary>
        /// Updates or deletes an existing comment.
        /// </summary>
        /// <param name="commentTicks">The existing comment creation ticks.</param>
        /// <param name="text">If null, the existing comment will be deleted.</param>
        /// <param name="itemKey">The commented item-key; if commenting Community, it may be left out for the main-community or specified with the sub-community.</param>
        /// <param name="itemType">The commented item-type for the item-key.</param>
        public void UpdateComment(long commentTicks, string text, string itemKey = null, GnCommentItemType itemType = GnCommentItemType.Community)
        {
            ExecuteCall("UpdateComment", new
            {
                commentTicks = commentTicks,
                itemType = (int)itemType,
                itemKey = itemKey,
                text = text
            }, cacheTtl: TimeSpan.MaxValue);

            ClearCacheForActionNames(CLEAR_CACHE_PATTERNS);
        }
    }
}
