﻿using System;
using System.Threading;
using System.Threading.Tasks;
using GpsNose.SDK.Mashup.Framework;
using GpsNose.SDK.Mashup.Model;

namespace GpsNose.SDK.Mashup.Api.Modules
{
    /// <summary>
    /// The main entry point for all other use-case specific API modules.
    /// The GnLoginApi instance is read by <see cref="GnApi.GetLoginApiForEndUser"/> or <see cref="GnApi.GetLoginApiForAdmin"/>.
    /// </summary>
    /// <remarks>
    /// The GnLoginApi can be in a state logged-in/out (<see cref="IsLoggedIn"/>) and can be user/admin type (<see cref="IsMashupAdmin"/>).
    ///
    /// Some API calls can be called only as a mashup admin, some needs an end-user login and some can be called as an anonymous guest user.
    ///
    /// The login can be used either once passively or fully GpsNose based actively:
    /// 1. Passively
    /// When your user logs-in once, you can take the <see cref="GnLogin.LoginName"/> for your own application specific user-management.
    /// You don't need to check the <see cref="CheckIfIsStillLoggedId"/> or care about time-to-live etc.
    ///
    /// 2. Actively
    /// You can fully rely on GpsNose user-management and don't store locally the available <see cref="GnLogin"/> data.
    /// In such case, you have to make sure the loginId you use is valid throughout the login-session, calling <see cref="CheckIfIsStillLoggedId"/>.
    /// </remarks>
    public abstract class GnLoginApiBase : GnApiModuleBase
    {
        protected override string ControllerBasePath => "Login";

        internal GnApi Api;
        internal string AppKey { get; }

        protected bool IsMashupAdmin => this is GnLoginApiAdmin;

        /// <summary>
        /// Holds if this instance was already successfully logged-in or not yet (i.e. guest).
        /// </summary>
        public bool? IsLoggedIn { get; private set; }

        /// <summary>
        /// The login-id denotes a concrete login-session. If generated by the lib, it's GUID. Otherwise it's the 
        /// </summary>
        public string LoginId { get; }

        /// <summary>
        /// The 2-letter lowercase language-id for language-specific server results.
        /// </summary>
        /// <remarks>
        /// Some API-calls results are language specific, like for example the items' keywords.
        /// </remarks>
        /// <example>
        /// "en"
        /// </example>
        public string LangId { get; }

        internal GnLoginApiBase(GnApi api, string appKey, string loginId, string langId)
        {
            IsLoggedIn = null;

            Api = api;
            AppKey = appKey;

            LoginId = loginId ?? Guid.NewGuid().ToString();

            // language must be supplied and will be only the 2-letter lowercase ISO-code
            if (string.IsNullOrWhiteSpace(langId) || langId.Length < 2)
                throw new GnException("langId must be supplied");

            LangId = langId.Substring(0, 2).ToLower();
        }



        /// <summary>
        /// Makes sure this GnLoginApi is already logged-in.
        /// </summary>
        public virtual void AssureLoggedInAlready()
        {
            if (IsLoggedIn == null)
                GetVerified();

            if (IsLoggedIn != true)
                throw new GnException($"no user with loginId={LoginId} is logged-in yet");
        }


        /// <summary>
        /// Returns a QR-code image for logging in.
        /// </summary>
        /// <param name="community">The target mashup community. For mashup-admin, it's the %www.gpsnose.com community.</param>
        /// <param name="mustJoin">If the user logging-in must first join the community.</param>
        /// <param name="needsActivation">If only activated users are allowed.</param>
        /// <param name="acls">What additional params should be returned from the platform.</param>
        /// <returns>Normally, only the user login-name. Additional params are possible by the ACLs.</returns>
        protected virtual byte[] GenerateQrCode(string community = GnSettings.GPSNOSE_COMMUNITY, bool mustJoin = false, bool needsActivation = false, GnMashupLoginAcl acls = GnMashupLoginAcl.None)
        {
            //if (string.IsNullOrWhiteSpace(community))
            //    throw new ArgumentException("community required");

            if (IsMashupAdmin)
            {
                if (community != GnSettings.GPSNOSE_COMMUNITY)
                    throw new GnException($"community for mashup-admin must be {GnSettings.GPSNOSE_COMMUNITY}");
            }
            else
            {
                if (community == GnSettings.GPSNOSE_COMMUNITY)
                    throw new GnException($"community for mashup-user must be not {GnSettings.GPSNOSE_COMMUNITY}");
            }

            var res = ExecuteCall("GenerateQrCode", new
            {
                loginId = LoginId,
                community = community?.Trim().ToLower(),
                activation = needsActivation,
                mustJoin = mustJoin,
                acls = ((int)acls).ToString()
            });

            var qrCodeBytes = (byte[])res;

            return qrCodeBytes;
        }


        /// <summary>
        /// Returns the login-verification data for this instance/loginId.
        /// </summary>
        /// <remarks>
        /// The data provided in GnLogin depends on which flags has been set in the generated QR-code when <see cref="GenerateQrCode"/> was called.
        /// </remarks>
        /// <returns>The login-data or null if not logged-in yet.</returns>
        public virtual GnLogin GetVerified()
        {
            // mark as already-called
            IsLoggedIn = false;

            var gnLoginResult = ExecuteCall<GnLogin>("GetVerified", new { loginId = LoginId }, true, TimeSpan.MaxValue);

            if (gnLoginResult != null)
                IsLoggedIn = true;

            return gnLoginResult;
        }


        /// <summary>
        /// Checks the logged-in state for this instance/loginId.
        /// </summary>
        /// <remarks>
        /// This call touches the sliding-expiration of the logged-in loginId.
        /// The TTL is 30 minutes.
        /// When you need to keep the logged-in state, you can call this once every 25 minutes or so.
        /// </remarks>
        /// <returns>True if still/already logged-in or False when not yet/anymore.</returns>
        public virtual bool CheckIfIsStillLoggedId()
        {
            var gnLoginResult = ExecuteCall<bool>("IsStillLoggedIn", new { loginId = LoginId }, cacheTtl: TimeSpan.MaxValue);

            return gnLoginResult;
        }


        /// <summary>
        /// Waits async for successful login in an own thread. The successful login comes from scanning the generated QR-code (<see cref="GenerateQrCode"/>) by the mobile app.
        /// </summary>
        /// <param name="cancellationToken">If provided, the wait can be time-out|cancelled by the caller.</param>
        /// <returns>The login data, which contain those fields specified in th QR-code options.</returns>
        /// <example>
        /// // get the API entry-point
        /// var api = new GnApi();
        ///
        /// // get the login-API: the appKey must be kept secret at your server!
        /// var userLoginApi = api.GetLoginApi(apiKey);
        ///
        /// // generate a QR-code for the end-user to login with his mobile device GpsNose app
        /// var qrCodeImageBytes = userLoginApi.GenerateQrCode("%www.geohamster.com");
        /// var qrCodeImage = Image.FromStream(new MemoryStream(qrCodeImageBytes));
        ///
        /// // show the QR-image somehow to the user: HTML img-tag, or WinForms' PictureBox etc.
        /// loginForm.qrCode = qrCodeImage;
        ///
        /// // you can optionally cancel the wait-for-login with a timeout or explicitly calling cancelSource.Cancel() on user's request
        /// var cancelSource = new CancellationTokenSource(5 * 60 * 1000);
        ///
        /// // wait for the user to log-in with his mobile device GpsNose app
        /// var loginTask = _userLoginApi.WaitForLogin(cancelSource.Token);
        /// try
        /// {
        ///     await loginTask;
        /// }
        /// catch (OperationCanceledException)
        /// {
        ///     // cancelled by time-out or user-cancelled: do nothing..
        ///     return;
        /// }
        ///
        /// // loginTask.Result now holds the loginName and optionally all the data you requested in the <see cref="GenerateQrCode"/> options
        /// </example>
        public virtual async Task<GnLogin> WaitForLogin(CancellationToken cancellationToken = new CancellationToken())
        {
            GnLogin login = GetVerified();

            var loginTask = Task.Run(() =>
            {
                while (IsLoggedIn != true)
                {
                    login = GetVerified();
                    if (login != null)
                        break;

                    cancellationToken.ThrowIfCancellationRequested();

                    Thread.Sleep(2000);

                    Console.WriteLine(Thread.CurrentThread.ManagedThreadId);
                }
            });

            await loginTask;

            return login;
        }
    }
}
