﻿using System;
using System.Collections.Generic;
using GpsNose.SDK.Mashup.Framework;
using GpsNose.SDK.Mashup.Model;

namespace GpsNose.SDK.Mashup.Api.Modules
{
    /// <summary>
    /// Messages receiving/sending API.
    /// </summary>
    /// <remarks>
    /// When no user is logged in, the community's creator is assumed as the mailing user.
    /// </remarks>
    public class GnMailsApi : GnApiModuleBase
    {
        private static readonly string[] CLEAR_CACHE_PATTERNS = { "SendMail" };


        internal GnMailsApi(GnLoginApiBase loginApi) : base(loginApi)
        {
        }


        /// <summary>
        /// Get the community-creator's mails.
        /// </summary>
        /// <remarks>
        /// Community-targeted mails are not included, only those where community-creator is explicitly included in mail's TO.
        /// Returning community-creator's mails, allows for unattended mail-automation workflows on your server.
        /// User's HasNewMails flag not touched: new-mails can be seen as usual in the mobile app.
        /// When a group-TO was used, you won't see it: you only known, that YOU are involved.
        /// </remarks>
        /// <param name="lastKnownTicks">The oldest known (already received) mail-ticks.</param>
        /// <param name="pageSize">The maximum mails to be returned in one page. Default max is 20 items.</param>
        /// <returns>The mails page of the community-creator.</returns>
        public List<GnMail> GetMailsPage(long lastKnownTicks = 0, int? pageSize = null)
        {
            var result = ExecuteCall<List<GnMail>>("GetMailsPage", new
            {
                lastKnownTicks = lastKnownTicks,
                pageSize = pageSize,
            });

            return result;
        }


        /// <summary>
        /// Send a message from the logged-in user - or the community-creator, to the receiver.
        /// </summary>
        /// <param name="toLoginName">To whom to send the message. The receiving user MUST have joined already your main or sub-community.</param>
        /// <param name="mailBody">The message body.</param>
        /// <remarks>
        /// Sending to an explicit group-of-logins is not allowed.
        /// However, you can send the message to your main- or sub-community, like: toLoginName = "%www.geohamster.com"
        /// You might want to prohibit sending (SPAM..) mails to whole sub/community from your users.
        /// If you're careful (controlled UI), user sending to sub/community is also a chat-technique.
        /// </remarks>
        public void SendMail(string toLoginName, string mailBody)
        {
            var creation = ExecuteCall("SendMail", new
            {
                toLoginName = toLoginName,
                mailBody = mailBody
            }, cacheTtl: TimeSpan.MaxValue);

            ClearCacheForActionNames(CLEAR_CACHE_PATTERNS);
        }

    }
}
