﻿using System;
using System.Collections.Generic;
using GpsNose.SDK.Mashup.Model;

namespace GpsNose.SDK.Mashup.Api.Modules
{
    /// <summary>
    /// Use this API to generate and read the user-scanned QR-tokens for your mashup site.
    /// </summary>
    /// <remarks>
    /// These QR-tokens are general-purpose, i.e. you generate a "token", which is an QR-code.
    /// Such QR-code can be printed/sent somewhere, so your community-members can scan it.
    /// Later, your website service pulls these tokens and get who/where/when/what was scanned.
    /// Imagine any membership-workflows, geo-games, check-ins in the field etc.
    ///
    /// Notice: never give your app-key away, as it can be used for reading the scanned tokens.
    /// The code-generation API call needs a valid user-login (so the later scanned-code has this creator-information),
    /// but the reading service has no user-login when calling the API, only the app-key, when reading the scanned-tokens data.
    /// </remarks>
    public class GnMashupTokensApi : GnApiModuleBase
    {
        /// <summary>
        /// User-scannable QR-tokens.
        /// </summary>
        /// <param name="loginApi"></param>
        internal GnMashupTokensApi(GnLoginApiBase loginApi) : base(loginApi)
        {
        }

        /// <summary>
        /// Generates a user-scannable QR-token.
        /// </summary>
        /// <param name="payload">Any custom data, which can be read-in as scanned-tokens by the mashup-site.</param>
        /// <param name="validToTicks">Until when is the token valid; default=0: forever</param>
        /// <param name="options">App-options for the UI when scanning this token.</param>
        /// <returns>The QR-image.</returns>
        /// <remarks>
        /// The QR-token contains encrypted token-data, which when scanned, are save to the GpsNose platform.
        /// The mashup site then can read all the scanned user-tokens back, to get who/where/when/what was scanned.
        /// </remarks>
        public byte[] GenerateQrTokenForMashup(
            string payload, 
            long validToTicks = 0, 
            float valuePerUnit = 0, 
            string label = null, 
            GnMashupTokenOptions options = GnMashupTokenOptions.NoOptions)
        {
            var buf = ExecuteCall<byte[]>("GenerateQrTokenForMashup", new
            {
                payload,
                validToTicks,
                valuePerUnit,
                label,
                options = (int)options
            }, cacheTtl: TimeSpan.FromDays(1));

            return buf;
        }


        public string GenerateQrTokenForMashupAsTextLink(
            string payload, 
            long validToTicks = 0, 
            float valuePerUnit = 0, 
            string label = null, 
            GnMashupTokenOptions options = GnMashupTokenOptions.NoOptions)
        {
            var buf = ExecuteCall<string>("GenerateQrTokenForMashup", new
            {
                payload,
                validToTicks,
                valuePerUnit,
                label,
                options = (int)options,
                format = (int)GnMashupTokenFormat.TextLink
            }, cacheTtl: TimeSpan.FromDays(1));

            return buf;
        }


        /// <summary>
        /// Get a page of user-scanned QR-tokens, from the oldest already read toward the newer ones.
        /// </summary>
        /// <param name="tag">The main- or sub-community tag for the tokens coming from. Default: the main community.</param>
        /// <param name="lastKnownScanTicks">The last (oldest) already known scanned token.</param>
        /// <param name="pageSize">How many tokens to return in one page. Default is max: 50</param>
        /// <returns></returns>
        public List<GnMashupToken> GetMashupTokensPage(string tag, long lastKnownScanTicks = 0, int? pageSize = null)
        {
            var tokens = ExecuteCall<List<GnMashupToken>>("GetMashupTokensPage", new
            {
                tag,
                lastKnownScanTicks,
                pageSize
            });

            return tokens;
        }


    }
}
