﻿using System;
using System.Collections.Generic;
using GpsNose.SDK.Framework.Json;
using GpsNose.SDK.Mashup.Model;

namespace GpsNose.SDK.Mashup.Api.Modules
{
    /// <summary>
    /// Storage reading/writing API.
    /// </summary>
    /// <remarks>
    /// Use this general storage API for saving/retrieving any kind of data items you need.
    /// Ever item belongs to a mashup-site and storage-name and is owned by a specific user.
    /// When saving/loading, you can impersonate an owner, as long as you are logged-in as the mashup-creator.
    /// </remarks>
    public class GnStorageApi : GnApiModuleBase
    {
        private static readonly string[] CLEAR_CACHE_PATTERNS = { "PutStorageItem" };


        internal GnStorageApi(GnLoginApiBase loginApi) : base(loginApi)
        {
        }


        /// <summary>
        /// Gets the raw storage-items page from a specific storage for an user.
        /// </summary>
        /// <remarks>
        /// You can use this raw paging version when needing the internal JSON data and/or the UpdatedTicks.
        /// </remarks>
        public List<GnMashupStorageItem> GetStorageItemsPage(string storage, int? pageSize = null, string lastKnownKey = null, string ownerUserName = null)
        {
            var result = ExecuteCall<List<GnMashupStorageItem>>("GetStorageItemsPage", new
            {
                storage = storage,
                pageSize = pageSize,
                lastKnownKey = lastKnownKey,
                ownerUserName = ownerUserName
            });

            return result;
        }


        /// <summary>
        /// Gets the storage-items page from a specific storage for an user.
        /// </summary>
        /// <returns>
        /// List of Tuple with key-value pairs.
        /// </returns>
        public List<Tuple<string, TItem>> GetStorageItemsPage<TItem>(string storage, int? pageSize = null, string lastKnownKey = null, string ownerUserName = null)
        {
            var rawPage = GetStorageItemsPage(storage, pageSize, lastKnownKey, ownerUserName);

            var page = new List<Tuple<string, TItem>>();
            var ser = new JsonTool();

            // convert the raw items into the deserialized typed ones
            foreach (var rawItem in rawPage)
            {
                var item = ser.DeserializeFromString<TItem>(rawItem.Value);
                page.Add(new Tuple<string, TItem>(rawItem.Key, item));
            }

            return page;
        }


        /// <summary>
        /// Gets a storage-item from a specific storage for an user by a given key.
        /// </summary>
        /// <remarks>
        /// Use this call to retrieve the internal value, possible with json inside.
        /// It's useful when needing the UpdatedTicks value for example.
        /// </remarks>
        public GnMashupStorageItem GetStorageItem(string storage, string key, string ownerUserName = null)
        {
            var result = ExecuteCall<GnMashupStorageItem>("GetStorageItem", new
            {
                storage = storage,
                key = key,
                ownerUserName = ownerUserName
            });

            return result;
        }


        /// <summary>
        /// Gets a storage-item from a specific storage for an user by a given key.
        /// </summary>
        /// <remarks>
        /// Use this call to retrieve the deserialized typed value from the internal JSON value.
        /// </remarks>
        public TItem GetStorageItem<TItem>(string storage, string key, string ownerUserName = null)
        {
            var result = GetStorageItem(storage, key, ownerUserName);

            var value = new JsonTool().DeserializeFromString<TItem>(result.Value);

            return value;
        }


        /// <summary>
        /// Puts/deletes a storage-item into/from a specific storage for an user.
        /// </summary>
        /// <param name="storage">The storage name: use this as you like, it's a kind of "table" name for organizing your key/values for an user.</param>
        /// <param name="key">The item's key: most non-special chars can be used.</param>
        /// <param name="value">The item's value or null: anything can be used, but the JSON serialized value should be less than approx 60KB.</param>
        /// <param name="ownerUserName">Whom the item will belong; default is the API caller. Only the mashup-creator is allowed to override the owner's name.</param>
        /// <remarks>To delete an item, set the value to null.</remarks>
        public void PutStorageItem(string storage, string key, object value, string ownerUserName = null)
        {
            // if the value is already a string, leave it as-is; otherwise, get the json
            string json = new JsonTool().SerializeToString(value);

            ExecuteCall("PutStorageItem", new
            {
                storage = storage,
                key = key,
                value = json,
                ownerUserName = ownerUserName
            });

            ClearCacheForActionNames(CLEAR_CACHE_PATTERNS);
        }

    }
}
