﻿using System.Web;
using GpsNose.SDK.Mashup.Framework;

namespace GpsNose.SDK.Mashup
{
    /// <summary>
    /// Holds the publicly accessible web paths to GpsNose.
    /// You can use override home and data access paths if local simulation is needed.
    /// </summary>
    public static class GnPaths
    {
        /// <summary>
        /// The home-page URL.
        /// </summary>
        public static string HomeUrl { get; set; } = "http://www.gpsnose.com";
        
        /// <summary>
        /// The data-page URL.
        /// </summary>
        public static string DataUrl { get; set; } = "http://data.gpsnose.com";

        /// <summary>
        /// Internal SDK development servers. Do not use - or configure as needed ;-).
        /// </summary>
        public static void SwitchToLocalEmulator()
        {
            DataUrl = "http://localhost:10000";
            HomeUrl = "http://localhost:1199";
        }

        /// <summary>
        /// For the real home-page, sometimes SSL is required. Use this prop accessing it.
        /// </summary>
        public static string HomeUrlSslNeeded
        {
            get
            {
                string url = HomeUrl;
                if (url.StartsWith("http://www.gpsnose.com"))
                    url = url.Replace("http://", "https://");

                return url;
            }
        }


        /// <summary>
        /// Gets the details-page at www.gpsnose.com for an item-type.
        /// </summary>
        /// <param name="itemType">The type of item the itemKey relates to.</param>
        /// <param name="itemKey">The unique-key of the given item. If null, it can be used for link-construction.</param>
        /// <returns></returns>
        public static string GetUrlForDetails(GnUrlItemType itemType, string itemKey = null)
        {
            switch (itemType)
            {
                case GnUrlItemType.Community: return $"{HomeUrl}/community/index/?profileTag={itemKey}";
                case GnUrlItemType.Event: return $"{HomeUrl}/event/detail/{itemKey}";
                case GnUrlItemType.Impression: return $"{HomeUrl}/impression/detail/{itemKey}";
                case GnUrlItemType.Nose: return $"{HomeUrl}/{itemKey}";
                case GnUrlItemType.PoI: return $"{HomeUrl}/poi/detail/{itemKey}";
                case GnUrlItemType.Track: return $"{HomeUrl}/tour/detail/{itemKey}";
            }

            throw new GnException($"unexpected itemType={itemType}");
        }


        /// <summary>
        /// Returns an image-url from www.gpsnose.com for an item.
        /// </summary>
        /// <param name="itemType">The item's type.</param>
        /// <param name="itemKey">The item's unique-key. If null, it returns the base-path for link-construction.</param>
        /// <param name="sizeType"></param>
        /// <returns></returns>
        public static string GetUrlForImage(GnUrlItemType itemType, string itemKey = null, GnThumbnailSize sizeType = GnThumbnailSize.Size_Full)
        {
            int size = (int)sizeType;
            string sizeSuffix = size == 0 ? "" : $"@{(int)sizeType}";

            switch (itemType)
            {
                case GnUrlItemType.Community: return $"{DataUrl}/commimg/{itemKey}{sizeSuffix}";
                case GnUrlItemType.Event: return $"{DataUrl}/eventsimg/{itemKey}{sizeSuffix}";
                case GnUrlItemType.Impression: return $"{DataUrl}/impression/{itemKey}{sizeSuffix}";
                case GnUrlItemType.Nose: return $"{DataUrl}/profimg/{itemKey}{sizeSuffix}";
                case GnUrlItemType.PoI: return $"{DataUrl}/locimg/{itemKey}{sizeSuffix}";
                
                // track has no image
                case GnUrlItemType.Track: break;
            }

            throw new GnException($"unexpected itemType={itemType}");
        }


        /// <summary>
        /// For IFRAME inclusion: News in creational order.
        /// </summary>
        /// <param name="lang">UI language.</param>
        /// <param name="community">For which community.</param>
        /// <returns>The URL for IFRAME src</returns>
        public static string NewsFrame(string lang, string community)
        {
            var pars = HttpUtility.ParseQueryString("");

            pars["hideNoses"] = "true";
            pars["isMashup"] = "true";

            if (!string.IsNullOrWhiteSpace(lang))
                pars["lang"] = lang;

            if (!string.IsNullOrWhiteSpace(community))
                pars["community"] = community.Trim().ToLower();

            string querySuffix = pars.ToString();
            if (!string.IsNullOrEmpty(querySuffix))
                querySuffix = $"?{querySuffix}";

            string s = $"{HomeUrl}/Home/Index{querySuffix}";

            return s;
        }


        /// <summary>
        /// For IFRAME inclusion: Noses in creational order.
        /// </summary>
        /// <param name="lang">UI language.</param>
        /// <param name="community">For which community.</param>
        /// <returns>The URL for IFRAME src</returns>
        public static string NosesFrame(string lang, string community)
        {
            var pars = HttpUtility.ParseQueryString("");

            pars["hideNews"] = "true";
            pars["isMashup"] = "true";

            if (!string.IsNullOrWhiteSpace(lang))
                pars["lang"] = lang;

            if (!string.IsNullOrWhiteSpace(community))
                pars["community"] = community.Trim().ToLower();

            string querySuffix = pars.ToString();
            if (!string.IsNullOrEmpty(querySuffix))
                querySuffix = $"?{querySuffix}";

            string s = $"{HomeUrl}/Home/Index{querySuffix}";

            return s;
        }

    }
}