﻿using System;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;

using GpsNose.SDK.Framework;
using GpsNose.SDK.Framework.Json;
using GpsNose.SDK.Mashup;
using GpsNose.SDK.Mashup.Api;
using GpsNose.SDK.Mashup.Api.Modules;
using GpsNose.SDK.Mashup.Framework;
using GpsNose.SDK.Mashup.Model;
using GpsNose.SDK.Web.NetFramework.Login;


using $rootnamespace$.Models.GpsNoseController;


namespace $rootnamespace$.Controllers
{
    public static class GnControllerExtensions
    {
        public static GnLoginApiEndUser GnLoginApi(this HttpRequestBase request, bool isAnonymousOk)
        {
            var api = new GnApi();

            if (!request.IsAuthenticated)
            {
                if (!isAnonymousOk)
                    throw new Exception("not allowed, please login first");

                // ok, let's create a fresh anonymous GnLogiApi instance
                return api.GetLoginApiForEndUser(GnLocalSettings.AppKey, langId: GnLocalSettings.LangId);
            }

            // request is authenticated: verify it's still valid..
            var loginApi = api.GetLoginApiForEndUser(GnLocalSettings.AppKey, GnAuthentication.CurrentUser.LoginId, GnLocalSettings.LangId);
            var loginData = loginApi.GetVerified();

            if (loginData == null)
            {
                // invalid login, it's probably expired
                if (!isAnonymousOk)
                    throw new Exception("login expired, please login first");

                // login is invalid, but anonymous is ok as well => logout from FormsAuthentication
                GnAuthentication.Logout();
                loginApi = api.GetLoginApiForEndUser(GnLocalSettings.AppKey, langId: GnLocalSettings.LangId);
            }

            return loginApi;
        }


        public static void SetViewBagCommunity(this Controller controller, string communityTag = null)
        {
            var loginApi = controller.Request.GnLoginApi(true);

            var communityApi = loginApi.GetCommunityApi();

            // this gets the main mashup community for members
            // notice: you can request here another sub-community of your mashups
            var comm = communityApi.GetCommunity(communityTag).ToExpando();

            // to have the UI label as well
            comm.TagName = communityTag ?? GnLocalSettings.Community;
            comm.QrCodeJoinImage = new UrlHelper(controller.Request.RequestContext).Action($"GenerateQrCodeForCommunityJoin", "GpsNose", new { community = communityTag });

            // save this for JS-usage
            controller.ViewBag.CommunityJson = new MvcHtmlString(controller.GnJson((object)comm));
        }


        public static string GnJson(this Controller controller, object data)
        {
            return GnWebUtility.GetJson(data);
        }


        public static ActionResult GnJsonResult(this Controller controller, object data)
        {
            string json = GnWebUtility.GetJson(data);

            return new ContentResult
            {
                ContentType = "text/plain",
                Content = json,
                ContentEncoding = Encoding.UTF8
            };
        }

    }


    public class GpsNoseController : Controller
    {
        public ActionResult GenerateQrCode(string loginId, string community, bool? mustJoin, bool? needsActivation, GnMashupLoginAcl acls = GnMashupLoginAcl.None)
        {
            var api = new GnApi().GetLoginApiForEndUser(GnLocalSettings.AppKey, loginId, GnLocalSettings.LangId);

            var buf = api.GenerateQrCode(true, true, acls);
            var mem = new MemoryStream(buf);
            mem.Seek(0, SeekOrigin.Begin);

            var res = new FileStreamResult(mem, "image/jpeg");
            return res;
        }


        public ActionResult GenerateQrCodeForCommunityJoin(string community = null)
        {
            var api = Request.GnLoginApi(true);
            var communityApi = api.GetCommunityApi();
            var buf = communityApi.GenerateQrCodeForCommunityJoin(community);

            var mem = new MemoryStream(buf);
            mem.Seek(0, SeekOrigin.Begin);

            var res = new FileStreamResult(mem, "image/jpeg");
            return res;
        }


        public ActionResult Logout()
        {
            GnAuthentication.Logout();
            return Redirect(FormsAuthentication.DefaultUrl);
        }


        public ActionResult Login(string loginId, string returnUrl)
        {
            // initial call => no loginId so far => generate & show UI
            if (string.IsNullOrWhiteSpace(loginId))
            {
                // continue to original page if already logged-in
                if (Request.IsAuthenticated)
                {
                    string returnUrl2 = !string.IsNullOrWhiteSpace(returnUrl) ?
                        returnUrl :
                        FormsAuthentication.DefaultUrl;

                    return Redirect(returnUrl2);
                }

                return View(new LoginModel());
            }

            // loginId provided (called from ajax, polling verification success)
            var api = new GnApi();
            var gnLogin = api.GetLoginApiForEndUser(GnLocalSettings.AppKey, loginId, GnLocalSettings.LangId).GetVerified();
            if (gnLogin == null)
                return Json(new { IsOk = false });

            // login is successful
            var authCookie = GnAuthentication.GenerateAuthenticationCookie(gnLogin, loginId, 30, false);
            Response.Cookies.Add(authCookie);

            return Json(new { IsOk = true });
        }


        [HttpPost]
        public ActionResult VerificationOk(string loginId)
        {
            var api = new GnApi();
            var result = api.GetLoginApiForEndUser(GnLocalSettings.AppKey, loginId, GnLocalSettings.LangId).GetVerified();
            if (result != null)
                return Json(result.LoginName);

            return new HttpStatusCodeResult(HttpStatusCode.ServiceUnavailable);
        }


        public ActionResult Members()
        {
            this.SetViewBagCommunity();

            return View();
        }


        [HttpPost]
        public ActionResult PageMembers(int? pageSize, long lastKnownTicks, string profileTag)
        {
            // there could be already a valid user-login: try use it, to have the API-call login-scoped
            var loginApi = Request.GnLoginApi(true);

            var commApi = loginApi.GetCommunityApi();

            var members = commApi.GetMembersPage(lastKnownTicks, profileTag, pageSize);

            var items = members.ToList().Select(n => new
            {
                LoginName = n.Key,
                JoinTicks = $"{n.Value}"
            }).ToList();

            return this.GnJsonResult(items);
        }


        public ActionResult NearbyNoses()
        {
            this.SetViewBagCommunity();

            return View();
        }


        public ActionResult PageNearbyNoses(string community)
        {
            var loginApi = Request.GnLoginApi(true);

            var nearbyApi = loginApi.GetNearbyApi();
            var noses = nearbyApi.GetNosesAround(community);

            return this.GnJsonResult(noses);
        }


        public ActionResult NearbyEvents()
        {
            this.SetViewBagCommunity();

            return View();
        }


        public ActionResult PageNearbyEvents(string community)
        {
            var loginApi = Request.GnLoginApi(true);

            var nearbyApi = loginApi.GetNearbyApi();
            var events = nearbyApi.GetEventsAround(community);

            return this.GnJsonResult(events);
        }


        public ActionResult News()
        {
            ViewBag.CommunityTag = GnLocalSettings.Community;

            return View();
        }


        [HttpPost]
        public ActionResult PageNews(int? pageSize, long lastKnownTicks, string community)
        {
            var loginApi = Request.GnLoginApi(true);

            var newsApi = loginApi.GetNewsApi();

            var items = newsApi.GetNewsPage(community, pageSize, lastKnownTicks);

            return this.GnJsonResult(items);
        }


        public ActionResult Comments()
        {
            this.SetViewBagCommunity();

            return View();
        }


        [HttpPost]
        public ActionResult PageComments(string uniqueKey, GnCommentItemType itemType, string lastKnownTicks, int? pageSize)
        {
            var api = new GnApi();

            long ticks = string.IsNullOrWhiteSpace(lastKnownTicks) ? GnSettings.FAR_FUTURE_TICKS : long.Parse(lastKnownTicks);

            var loginApi = Request.GnLoginApi(true);

            var commentsApi = loginApi.GetCommentsApi();

            var items = commentsApi.GetCommentsPage(uniqueKey, itemType, pageSize, ticks);

            return this.GnJsonResult(items);
        }


        [HttpPost, Authorize]
        public ActionResult SaveComment(bool isUpdate, string uniqueKey, string text, GnCommentItemType itemType = GnCommentItemType.Community, long? creationTicks = null)
        {
            var loginApi = Request.GnLoginApi(false);

            var commentsApi = loginApi.GetCommentsApi();

            if (!isUpdate)
            {
                // ADD
                creationTicks = commentsApi.AddComment(text, uniqueKey, itemType);
                return this.GnJsonResult(creationTicks);
            }
            else
            {
                // UPDATE
                commentsApi.UpdateComment((long)creationTicks, text, uniqueKey, itemType);
                return this.GnJsonResult(null);
            }
        }

    }
}
